// ======================================================================
//
// ObjectList.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef OBJECT_LIST_H
#define OBJECT_LIST_H

// ======================================================================

const bool OBJLIST_DEFAULT_WRAP   = true;
const bool OBJLIST_DEFAULT_DELETE = true;

// ======================================================================

class Camera;
class Object;

// ======================================================================

class ObjectList
{
private:

	// rendering information
	int          depth;
	bool         resize;

	// the actual list information
	int      maxObjects;
	int      numObjects;
	Object **object;

	// spare list for use while in alter routine	
	int      alterNum;
	Object **alterSafe;

private:

	void full(void);
	bool findSlot(int uniqueId, int *slot) const;
	void remove(const Object *obj, bool del, bool removeFromAlterSafe);

private:

	// disable the copy constructor and the assignment operator
	ObjectList(void);
	ObjectList(const ObjectList &);
	ObjectList &operator =(const ObjectList &);

public:

	explicit ObjectList(int max, int depth=0, bool allowResize=false);
	~ObjectList(void);

	// look inside object list
	int           getMaxObjects(void) const;
	int           getNumObjects(void) const;
	Object       *operator [](int index);
	const Object *operator [](int index) const;
	Object       *getObject(int index);
	const Object *getObject(int index) const;

	// logical depth for object list merging at render time
	int     getDepth(void) const;

	// add and remove	objects
	void    add(Object *obj);
	void    remove(const Object *obj, bool del=OBJLIST_DEFAULT_DELETE);

	// search object list
	const Object *findNextObject(const Object *obj, bool wrap=OBJLIST_DEFAULT_WRAP) const;
	const Object *findPrevObject(const Object *obj, bool wrap=OBJLIST_DEFAULT_WRAP) const;
	const Object *findObject(int uniqueId) const;

	// delete all items on the object list
	void    removeAll(bool del=OBJLIST_DEFAULT_DELETE);
	
	// call this routine in all objects in the object list
	void    render(void);
	void    prepareToAlter(void);
	void    alter(real time);
	void    addToCamera(Camera &camera) const;
};

// ======================================================================

inline int ObjectList::getDepth(void) const
{
	return depth;
}

// ----------------------------------------------------------------------

inline int ObjectList::getNumObjects(void) const
{
	return numObjects;
}

// ----------------------------------------------------------------------

inline int ObjectList::getMaxObjects(void) const
{
	return maxObjects;
}

// ----------------------------------------------------------------------

inline void ObjectList::remove(const Object *obj, bool del)
{
	remove(obj, del, true);
}

// ----------------------------------------------------------------------

inline Object *ObjectList::operator[](int index)
{
	return getObject(index);
}

// ----------------------------------------------------------------------

inline const Object *ObjectList::operator[](int index) const
{
	return getObject(index);
}

// ======================================================================

#endif
